#ifndef VELODYNE_TARGET_H
#define VELODYNE_TARGET_H

/* Internal headers */
#include "server/velodyneHeader.h"
#include "velodyneClient.h"

/* Directives */
#define VELODYNE_DEFAULT_MAX_NUM_TARGETS 512
#define VELODYNE_MAX_TARGET_POINTS 4000

/* Structures */
typedef enum velodyneTargetStatus {
  VELODYNE_TARGET_NOT_DETECTED,
  VELODYNE_TARGET_NOT_ENOUGH_POINTS,
  VELODYNE_TARGET_NOT_WIDE_ENOUGH,
  VELODYNE_TARGET_TOO_WIDE,
  VELODYNE_TARGET_NOT_HIGH_ENOUGH,
  VELODYNE_TARGET_TOO_HIGH,
  VELODYNE_TARGET_DETECTED
} velodyneTargetStatus;

typedef struct velodyneTarget {
  velodyneTargetStatus status;
  int                  detectedTargetPoints;
  float                meanReflectance;
  double               radius;
  velodyne3DPoint      targetPoints[VELODYNE_MAX_TARGET_POINTS];
  velodyne3DPoint      max;
  velodyne3DPoint      min;
  velodyne3DPoint      cog;
  velodyne3DPoint      stdDev;
  /* These 3 points are used to store the min and max coordinates of the point
     set of the target, its center of gravity (i.e. mean) and standard deviation
     -- it might be a bit awkward to use the velodyne3DPoint structure for these
     purposes, but it is a simple way to do. */
} velodyneTarget;

typedef struct velodyneTargetTracking {
  int            numTargets;
  velodyneTarget targets[VELODYNE_DEFAULT_MAX_NUM_TARGETS];
} velodyneTargetTracking;

/* Results */
typedef enum velodyneTargetResult {
  VELODYNE_TARGET_OK,
  VELODYNE_TARGET_ERR_CANNOT_OPEN_FILE,
  VELODYNE_TARGET_ERR_CANNOT_WRITE_TARGET
} velodyneTargetResult;

/* Functions */
ACTIVITY_EVENT velodyne_analyzeTarget(
    velodyneTargetParams*, velodyneTargetDetectionParams*, velodyneTarget*,
    const velodyneTargetDetectionFileOutput* fileOutput);
void velodyne_eraseTargetPoints(velodyne3DImage*, velodyneTarget*);
void velodyneTrackTarget(
    velodyneTarget* target,
    velodyneTargetTracking* targetTracking,
    velodyneTargetTrackingParams* targetTrackingParams,
    velodyneTargetTrackingFileOutputParams* targetTrackingFileOutputParams,
    float targetCOGX,
    float targetCOGY,
    int* report);

#endif
